<?php

namespace FiloBlu\Refilo\Model\Config\Seo;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class HrefLangConfig
 * @package FiloBlu\Refilo\Model\Config\Seo
 */
class HrefLangConfig
{

    const REFILO_SEO_HREFLANG_ENABLED = 'mageworx_seo/base/hreflangs/enabled';
    const REFILO_SEO_HREFLANG_ENABLED_CATEGORY = 'mageworx_seo/base/hreflangs/enabled_category';
    const REFILO_SEO_HREFLANG_ENABLED_PRODUCT = 'mageworx_seo/base/hreflangs/enabled_product';
    const REFILO_SEO_HREFLANG_ENABLED_CMS = 'mageworx_seo/base/hreflangs/enabled_cms';
    const REFILO_SEO_HREFLANG_LANG_CODE = 'mageworx_seo/base/hreflangs/lang_code';
    const REFILO_SEO_HREFLANG_COUNTRY_CODE_ENABLED = 'mageworx_seo/base/hreflangs/country_code_enabled';
    const REFILO_SEO_HREFLANG_COUNTRY_CODE = 'mageworx_seo/base/hreflangs/country_code';
    const REFILO_SEO_HREFLANG_USE_MAGENTO_LANG_CODE = 'mageworx_seo/base/hreflangs/use_magento_lang_code';
    const REFILO_SEO_HREFLANG_ENABLED_HOME = 'refilo_content/general/enable_hreflang_home';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * HrefLangConfig constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    )
    {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param StoreManagerInterface $store
     * @return mixed|string
     */
    public function getSeoLocaleCodeForStore($store)
    {
        $locale = $this->scopeConfig->getValue(self::REFILO_SEO_HREFLANG_LANG_CODE, ScopeInterface::SCOPE_STORE, (int)$store->getId());
        if (empty($locale)) {
            $locale = $this->scopeConfig->getValue(self::REFILO_SEO_HREFLANG_LANG_CODE, ScopeInterface::SCOPE_STORE, 0);
        }
        $addCountryCode = $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_COUNTRY_CODE_ENABLED, ScopeInterface::SCOPE_STORE, (int)$store->getId());
        if ($addCountryCode) {
            $locale .= '-' . $this->scopeConfig->getValue(self::REFILO_SEO_HREFLANG_COUNTRY_CODE, ScopeInterface::SCOPE_STORE, (int)$store->getId());
        }
        return $locale;
    }

    /**
     * @param int $storeId
     * @return array
     */
    public function getConfig($store)
    {
        return [
            'enabled'               => $this->isHrefLangEnabled($store->getId()),
            'enabled_category'      => $this->isHrefLangEnabledForCategory($store->getId()),
            'enabled_product'       => $this->isHrefLangEnabledForProduct($store->getId()),
            'enabled_cms'           => $this->isHrefLangEnabledForCms($store->getId()),
            'enabled_home'          => $this->isHrefLangEnabledForHome($store->getId()),
            'use_magento_lang_code' => $this->getUseMagentoLangCode($store->getId())
        ];
    }

    /**
     * @param int $storeId
     * @return bool
     */
    public function isHrefLangEnabled($storeId = 0)
    {
        return $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_ENABLED, ScopeInterface::SCOPE_STORE, (int)$storeId);
    }

    /**
     * @param int $storeId
     * @return bool
     */
    public function isHrefLangEnabledForCategory($storeId = 0)
    {
        return $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_ENABLED_CATEGORY, ScopeInterface::SCOPE_STORE, (int)$storeId);
    }

    /**
     * @param int $storeId
     * @return bool
     */
    public function isHrefLangEnabledForProduct($storeId = 0)
    {
        return $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_ENABLED_PRODUCT, ScopeInterface::SCOPE_STORE, (int)$storeId);
    }

    /**
     * @param int $storeId
     * @return bool
     */
    public function isHrefLangEnabledForCms($storeId = 0)
    {
        return $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_ENABLED_CMS, ScopeInterface::SCOPE_STORE, (int)$storeId);
    }

    /**
     * @param int $storeId
     * @return bool
     */
    public function getUseMagentoLangCode($storeId = 0)
    {
        return $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_USE_MAGENTO_LANG_CODE, ScopeInterface::SCOPE_STORE, (int)$storeId);
    }

    /**
     * @param int $storeId
     * @return bool
     */
    public function isHrefLangEnabledForHome($storeId = 0) {
        return $this->scopeConfig->isSetFlag(self::REFILO_SEO_HREFLANG_ENABLED_HOME, ScopeInterface::SCOPE_STORE, (int)$storeId);
    }
}
